import abc
import re
import os
import sys
import random
import uuid
import json as jsonlib
from dogpile.cache import make_region
from dogpile.cache.api import NO_VALUE
from .parsers.multipart import MultipartParser
from .dotdict import DotDict
from .ext.session import Session as Session
from .errors import AbortRender
from jinja2 import Environment, PackageLoader, ChoiceLoader, FileSystemLoader


class Middleware(object):
    def __init__(self, app, request, response):
        self.app = app
        self.request = request
        self.response = response

    def __enter__(self):
        if not self.response._aborted:
            try:
                return self.enter()
            except AbortRender:
                self.response._aborted = True

    def __exit__(self, err_type, err_obj, traceback):
        if not self.response._aborted:
            self.exit_arguments = (err_type, err_obj, traceback)
            try:
                self.exit()
            except AbortRender:
                self.response._aborted = True

        if isinstance(err_obj, AbortRender):
            self.response._aborted = True

    def enter(self):
        pass

    def exit(self):
        pass


def connection_helper(connection='close'):
    class ConnectionHelperMiddleware(Middleware):
        def exit(self):
            self.response.headers['Connection'] = connection
            self.response.headers['Content-Length'] = str(len(
                self.response.body))

    return ConnectionHelperMiddleware


def static(path, expose_as='/'):
    import mimetypes

    expose_as = (expose_as or os.path.basename(path)).strip('/')
    pattern = re.compile('^/%s.*' % re.escape(expose_as))
    mimetypes.init()

    class StaticMiddleware(Middleware):
        def enter(self):
            match = pattern.match(self.request.headers.path_info)
            local_path = None

            if match:
                local_path = self._get_local_path(
                    self.request.headers.path_info)
                if local_path:
                    self.response.set('Content-Type', mimetypes.guess_type(
                        local_path)[0])
                    self.response.status = 200
                    try:
                        with open(local_path, 'r') as f:
                            self.response.body = f.read()
                    except OSError, e:
                        pass
                    finally:
                        raise AbortRender

        def _get_local_path(self, local_path):
            local_path = local_path.strip('/')[len(expose_as):].lstrip('/')
            temp_path = os.path.join(path, local_path)
            if temp_path.startswith(path) and os.path.isfile(temp_path):
                return temp_path
            else:
                return None

    return StaticMiddleware


def cookie_parser():
    class CookieParserMiddleware(Middleware):
        def enter(self):
            try:
                data = self.request.cookies.split('; ')
            except AttributeError:
                self.request.cookies = DotDict()
                return

            self.request.cookies = DotDict()
            for i in data:
                if i:
                    split = i.split('=', 1)
                    if split:
                        self.request.cookies[split[0]] = split[1]

    return CookieParserMiddleware


def session(cache='gimme.cache.memory', session_cookie='gimme_session',
        make_session_key=uuid.uuid4, expiration_time=60*60*24*7, **kwargs):

    region = make_region().configure(cache,
        expiration_time=expiration_time, **kwargs)

    class SessionMiddleware(Middleware):
        def enter(self):
            self.request.session = self._load_session()

        def exit(self):
            if self.request.session._state.is_dirty():
                self.request.session.save()

        def _load_session(self):
            try:
                key = self.request.cookies[session_cookie]
            except KeyError:
                return self._create_session()

            session_data = region.get(key)

            if session_data != NO_VALUE:
                return Session(region, key, session_data)
            else:
                return self._create_session()

        def _create_session(self):
            key = str(make_session_key())
            self.response.set('Set-Cookie', '%s=%s' % (session_cookie, key))
            new_session = Session(region, key, {}, True)
            return new_session

    return SessionMiddleware


def json():
    class JsonMiddleware(Middleware):
        def enter(self):
            if not hasattr(self.request, 'body'):
                self.request.body = DotDict()
     
            if self.request.type == 'json':
                try:
                    parsed_data = jsonlib.loads(self.request.raw_body)
                except ValueError:
                    pass
                else:
                    for k, v in parsed_data.iteritems():
                        self.request.body[k] = v

    return JsonMiddleware


def urlencoded():
    from .uri import QueryString

    class URLEncodedMiddleware(Middleware):
        def enter(self):
            if not hasattr(self.request, 'body'):
                self.request.body = DotDict()
     
            if self.request.type == 'application/x-www-form-urlencoded':
                qs = QueryString(self.request.raw_body)
                for k, v in qs.iteritems():
                    self.request.body[k] = v

    return URLEncodedMiddleware


def multipart():
    multipart_pattern = re.compile('^multipart/form-data; boundary=(.*)', re.I)

    class MultipartMiddleware(Middleware):
        def enter(self):
            if not hasattr(self.request, 'body'):
                self.request.body = DotDict()

            if not hasattr(self.request, 'files'):
                self.request.files = DotDict()
                
            if ('content_type' in self.request.headers and
                    'request_method' in self.request.headers and
                    self.request.headers.request_method in ('PUT', 'POST')):

                match = multipart_pattern.match(
                    self.request.headers.content_type)

                if match:
                    mp = MultipartParser(match.group(1), self.request.wsgi.input)
                    for name, mp_file in mp.iteritems():
                        if mp_file.value:
                            self.request.body[name] = mp_file.value
                        else:
                            self.request.files[name] = mp_file

    return MultipartMiddleware


def body_parser(json_args={}, urlencoded_args={}, multipart_args={}):
    json_parser = json(**json_args)
    urlencoded_parser = urlencoded(**urlencoded_args)
    multipart_parser = multipart(**multipart_args)

    class BodyParserMiddleware(Middleware):
        def enter(self):
            json_parser(self.app, self.request, self.response).enter()
            urlencoded_parser(self.app, self.request, self.response).enter()
            multipart_parser(self.app, self.request, self.response).enter()

    return BodyParserMiddleware


def method_override():
    from .uri import QueryString

    multipart_pattern = re.compile('^multipart/form-data; boundary=(.*)', re.I)

    class MethodOverrideMiddleware(Middleware):
        def enter(self):
            if self.request.type == 'application/x-www-form-urlencoded':
                query_string = self.QueryString(self.request.raw_body)
                if '_method' in query_string:
                    self.request.headers.request_method = query_string._method

    return MethodOverrideMiddleware


def compress():
    import zlib

    class CompressMiddleware(Middleware):
        def exit(self):
            if 'accept_encoding' in self.request.headers:
                    if ('deflate' in
                            self.request.headers.accept_encoding.split(',')):
                        try:
                            compressed = zlib.compress(self.response.body)
                        except TypeError:
                            pass
                        else:
                            self.response.headers['Content-Encoding'] = 'deflate'
                            self.response.headers['Content-Length'] = str(
                                len(compressed))
                            self.response.body = compressed

    return CompressMiddleware


def profiler(fn=None, pr=None):
    import cProfile

    class ProfilerMiddleware(Middleware):
        def enter(self):
            if pr is None:
                pr = cProfile.Profile()

            pr.enable()

        def exit(self):
            pr.disable()
            if fn:
                fn(pr)

    return ProfilerMiddleware



class DotDict(dict):
    '''
    A simple subclass of :class:`dict` that allows for dot-notation access of
    all elements. This is used rather extensively through Frame to make it
    easier to access nested items. For example, request header data, response
    header data, and config directives are allow stored using this class.
    '''

    def __init__(self, data=None):
        new_data = dict(data) if data is not None else {}

        for k, v in new_data.iteritems():
            if isinstance(v, dict):
                new_data[k] = DotDict(v)

        dict.__init__(self, new_data)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError, e:
            raise AttributeError(e)

    def __setattr__(self, key, value):
        if isinstance(value, dict) and not isinstance(value, DotDict):
            self[key] = DotDict(value)
        else:
            self[key] = value



from . import response
from . import controller
from .parsers.status import StatusCode
import gimme.routes


class GimmeError(Exception):
    pass


class RouteError(GimmeError):
    pass


class TemplateError(GimmeError):
    pass


class AcceptFormatError(GimmeError):
    pass


class FDError(GimmeError):
    pass


class MultipartError(GimmeError):
    pass


class AbortRender(GimmeError):
    pass


class HTTPError(GimmeError):
    def __init__(self, status=500):
        self.status = StatusCode(status)
        try:
            method = getattr(controller.ErrorController, 'http%s'
                % self.status.code)
        except AttributeError:
            method = controller.ErrorController.generic

        self.route = gimme.routes.RouteMapping(None, '*', [], method)

    def make_response(self, request):
        res = response.Response(request.app, self.route, request)
        res.status = self.status.get()
        return res


class HTTPRedirect(HTTPError):
    def __init__(self, url, status=301):
        self.url = url
        HTTPError.__init__(self, status)

    def make_response(self, request):
        res = response.Response(request.app, self.route, request)
        res.status = self.status.get()
        res.redirect(self.url, self.status.code)
        return res



import re
from . import errors
from .request import Request
from .response import Response
from .controller import ErrorController


class PatternMatch(object):
    def __init__(self, pattern, match):
        self.pattern = pattern
        self.match = match


class RouteList(list):
    def __init__(self, data=[]):
        for i in data:
            if isinstance(i, Route):
                self.append(i)
            else:
                self.append(Route(i))

    def __repr__(self):
        return "<RouteList([%s])>" % ', '.join(map(repr, self))

    def __or__(self, other):
        self.append(other)
        return self

    def match(self, uri):
        for i in self:
            match = i.match(uri)
            if match:
                return match
        return None


class Route(object):
    __sub_pattern = re.compile(':([a-zA-Z_\-0-9]+)(\?)?')
    __sub_last_pattern = re.compile('\/:([a-zA-Z_\-0-9]+)(\?)?$')

    def __init__(self, regex):
        if isinstance(regex, str):
            self._regex = self._make_regex(regex)
        else:
            self._regex = regex

    def __repr__(self):
        return "<Route(%s)>" % self._regex.pattern

    def match(self, uri):
        match = self._regex.match(uri)
        return PatternMatch(self, match) if match else None

    def _make_regex(self, string):
        if string == '*':
            return re.compile('.*')

        def handle_replace(match):
            return '(?P<%s>[a-zA-Z0-9_\-\.,]+)%s' % (match.group(1),
                match.group(2) or '')

        def handle_last_replace(match):
            return ('(?P<__last>/)?(?(__last)(?P<{0}>[a-zA-Z0-9_\-\.,]+){1})'
                '(/)?').format(
                match.group(1),
                match.group(2) or '')

        pattern = self.__sub_last_pattern.sub(handle_last_replace, string)
        pattern = '^%s$' % self.__sub_pattern.sub(handle_replace, pattern)
        return re.compile(pattern)

    def __or__(self, other):
        return RouteList([self, other])


class RouteMapping(object):
    def __init__(self, routes, pattern, middleware, method, match_fn=None):
        self.routes = routes

        if isinstance(pattern, (list, tuple)):
            pattern = RouteList(pattern)
        else:
            pattern = Route(pattern)

        self.pattern = pattern
        self.middleware = middleware
        self.method = method
        self.match_fn = match_fn

    def match(self, environ):
        uri = environ[self.routes.match_param]

        if not self.match_fn or (self.match_fn and
                self.match_fn(uri, environ)):
            return self.pattern.match(uri)


class Routes(object):
    def __init__(self, app, match_param='PATH_INFO',
            strip_trailing_slash=True):
        self.app = app
        self.match_param = match_param
        self.strip_trailing_slash = strip_trailing_slash

        self.__get = []
        self.__post = []
        self.__put = []
        self.__delete = []
        self.__all = []

        self.http404 = RouteMapping(self, '*', [], ErrorController.http404)
        self.http500 = RouteMapping(self, '*', [], ErrorController.http500)

    def _add(self, routes_list, pattern, *args, **kwargs):
        middleware = list(args[:-1])
        fn = kwargs['fn'] if 'fn' in kwargs else None
        try:
            method = args[-1]
        except IndexError, e:
            raise errors.RouteError("No controller method specified.")
        routes_list.append(RouteMapping(self, pattern, middleware, method, fn))

    def get(self, pattern, *args, **kwargs):
        self._add(self.__get, pattern, *args, **kwargs)

    def post(self, pattern, *args, **kwargs):
        self._add(self.__post, pattern, *args, **kwargs)

    def put(self, pattern, *args, **kwargs):
        self._add(self.__put, pattern, *args, **kwargs)

    def delete(self, pattern, *args, **kwargs):
        self._add(self.__delete, pattern, *args, **kwargs)

    def all(self, pattern, *args, **kwargs):
        self._add(self.__all, pattern, *args, **kwargs)

    def _find_match(self, environ, match_list):
        for i in match_list:
            match = i.match(environ)
            if match:
                request = Request(self.app, environ, match)
                response = Response(self.app, i, request)
                return (request, response)

    def match(self, environ):
        request_methods = {
            'GET': self.__get,
            'POST': self.__post,
            'PUT': self.__put,
            'DELETE': self.__delete
        }

        request_method = environ['REQUEST_METHOD'].upper()

        if request_method in request_methods:
            match_list = request_methods[request_method]
            result = self._find_match(environ, match_list)
            if result:
                return result

        result = self._find_match(environ, self.__all)
        if result:
            return result

        request = Request(self.app, environ, None)
        response = Response(self.app, self.http404, request)
        return (request, response)



import threading
import sys
import time
import datetime
import os


class ModuleMonitor(threading.Thread):
    '''
    Monitors all of the modules loaded by the current running application. If
    any of them change (detected via os.stat), reload the running process.
    '''
    def __init__(self, server, interval=1):
        self.server = server
        self.interval = interval
        self._stop = threading.Event()

        threading.Thread.__init__(self)

    def run(self):
        self.server.logger.log_info("Starting module reloader...")
        datetime.datetime.now()
        last_scan = datetime.datetime.now()

        old_stats = self.stat_modules()

        while not self.stopped():
            now = datetime.datetime.now()
            delta = datetime.timedelta(seconds=self.interval)
            if now > last_scan + delta:
                new_stats = self.stat_modules()
                if not self.compare_stats(old_stats, new_stats):
                    self.server.logger.log_info(
                        "File changed, reloading server...")
                    self.server.stop()
                    self.restart_app()
                old_stats = new_stats
            time.sleep(1)

    def restart_app(self):
        python = sys.executable
        os.execl(python, python, *sys.argv)

    def stopped(self):
        return self._stop.is_set()

    def stop(self):
        self.server.logger.log_info("Stopping module reloader...")
        self._stop.set()

    def compare_stats(self, old, new):
        '''
        Returns true if the two stat dictionaries don't differ, false
        if they do.
        '''
        for key, value in old.iteritems():
            if key in new and new[key].st_mtime != value.st_mtime:
                return False
        return True

    def stat_modules(self):
        stats = {}
        for name, module in sys.modules.items():
            try:
                path = module.__file__
            except AttributeError:
                continue

            try:
                stats[path] = os.stat(path)
            except EnvironmentError:
                pass

            try:
                path = path[:-1]
                stats[path] = os.stat(path)
            except EnvironmentError:
                pass

        return stats



import traceback
import time
import datetime
import mimetypes
import re
import sys
from contextlib import nested
from .dotdict import DotDict
from .headers import ResponseHeaders, Header
from .controller import ErrorController
from .parsers.status import StatusCode
import gimme.errors


class Response(object):
    '''
    The Response class is responsible for aggregating all of the information
    required for generating a response and for rendering said response.

    :ivar app: The Gimme application.
    :ivar route: The route that matched the request/response objects.
    :ivar request: The request object.
    :ivar headers: An instance of :class:`gimme.headers.ResponseHeaders` that
        will eventually be converted to a string appropriate for the HTTP
        response headers.
    :ivar body: The body of response.
    :ivar locals: An instance of :class:`gimme.dotdict.DotDict` that is
        intended to store arbitrary information that is unique to a given
        response object.
    '''

    _charset_pattern = re.compile('(.*?); charset=(.*)$')
    mimetypes.init()

    def __init__(self, app, route, request):
        self.app = app
        self.route = route
        self.request = request
        self._status = StatusCode('200 OK')
        self._aborted = False

        try:
            self.headers = ResponseHeaders(dict(app.get('default headers')))
        except KeyError, e:
            self.headers = ResponseHeaders()

        self._controller_class = (route.method.im_class
            if hasattr(route.method, 'im_class') else None)
        self.body = None

        # Storage for request-specific local data
        self.locals = DotDict()

    @property
    def status(self):
        '''
        The response status code. This is an instance of
        :class:`StatusCode <gimme.parsers.status.StatusCode>` and can be set
        using any of the following methods::

            response.status = 404
            response.status = "404 Not Found"
            reponse.status.set(404)

        Checking a status is as simple as doing any of the following::

            response.status == 404
            response.status == "404 Not Found"
        '''
        return self._status

    @status.setter
    def status(self, status):
        self._status.set(status)

    def _render(self, middleware=None):
        if self._controller_class:
            controller = self._controller_class(self.app, self.request, self)
            method = self.route.method
        else:
            controller = None
            def method():
                return self.route.method

        if middleware is None:
            middleware = (self.app._middleware + self.route.middleware)
        elif not middleware:
            middleware = []

        instantiated_middleware = self._instantiate_middleware(middleware)

        with nested(*instantiated_middleware):
            try:
                if not self._aborted:
                    self.body = method()
            except gimme.errors.AbortRender:
                self._aborted = True

    def _instantiate_middleware(self, middleware):
        result = []
        for i in middleware:
            result.append(i(self.app, self.request, self))
        return result

    def set(self, key, value):
        '''
        A shortcut for setting a response header.

        :param key: The key of the header to set.
        :param value: The value of the header.
        '''
        self.headers[key] = value
        return self

    def get(self, key):
        '''
        A shortcut for getting a response header.

        :param key: The key of the header to get.
        '''
        return self.headers[key]

    def redirect(self, path, code=302):
        '''
        A shortcut for setting the response status and HTTP "Location" header
        appropriately to redirect the requesting client.

        :param str path: The location to send the client to.
        :param code: The status code to set.
        '''
        self.status = code
        self.location = path

    @property
    def location(self):
        '''
        A shortcut for getting and setting the HTTP "Location" header.
        '''
        return self.get('Location')

    @location.setter
    def location(self, path):
        self.set('Location', path)

    @property
    def type(self):
        '''
        A shortcut for getting and setting the HTTP "Content-Type" header.
        '''
        return self.get('Content-Type')

    @type.setter
    def type(self, content_type):
        self.set('Content-Type', content_type)

    def cookie(self, key, value, expires=None, http_only=False, secure=False,
            path='/', domain=None):
        '''
        A helper method for setting the HTTP "Set-Cookie" header.

        :param key: The key/name of the cookie.
        :param value: What to set the cookie to.
        :param expires: Number of seconds that the cookie should last before
            being expired. Defaults to ``None``, which makes the cookie
            last indefinitely.
        :param http_only: Set the HttpOnly flag.
        :param secure: Set the Secure flag.
        :param path: Limit the URI path that the cookie applies to.
        :param domain: Limit the domain that the cookie applies to.
        '''
        cookie_string = ['%s=%s' % (key, value)]

        if domain:
            cookie_string.append('Domain=%s' % domain)

        if path:
            cookie_string.append('Path=%s' % path)

        if expires:
            if isinstance(expires, int):
                date = datetime.datetime.utcfromtimestamp(time.mktime(
                    time.gmtime(time.time() + expires)))
            elif isinstance(expires, datetime.datetime):
                date = expires
            else:
                date = datetime.datetime.utcnow()
            cookie_string.append('Expires=%s' % date.strftime(
                '%a, %d %b %Y %H:%M:%S GMT'))

        if secure:
            cookie_string.append('Secure')

        if http_only:
            cookie_string.append('HttpOnly')

        self.headers.add_header(Header('Set-Cookie', '; '.join(cookie_string)))

    def clear_cookie(self, key):
        '''
        A helper method for expiring a previously-set cookie.

        :param key: The key/name of the cookie to expire.
        '''
        expires = datetime.datetime.fromtimestamp(0)
        self.cookie(key, 'deleted', expires=expires)

    def attachment(self, filename=None):
        '''
        A helper for setting the "Content-Disposition" and "Content-Type" HTTP
        headers for a given file. This makes it easy to pass a file to the
        client for downloading.

        Note, this is implemented as a setter-only property. In other words,
        you cannot "get" the attachment information with this property, but
        only set it.

        This is valid::

            response.attachment = "something.jpg"

        But this is **not**::

            print response.attachment
        '''
        if filename:
            self.headers['Content-Disposition'] = ('attachment; filename="%s"'
                % filename)
            mimetype, encoding = mimetypes.guess_type(filename)
            if mimetype:
                self.type = mimetype
        else:
            self.headers['Content-Disposition'] = 'attachment';

    attachment = property(None, attachment, None, attachment.__doc__)

    @property
    def charset(self):
        '''
        A helper for getting and setting the charset portion of the
        "Content-Type" header.
        '''
        if 'Content-Type' in self.headers:
            header = self.headers['Content-Type']
            match = self._charset_pattern.match(header.value)
            if match:
                return match.group(2)
        return None

    @charset.setter
    def charset(self, value):
        if 'Content-Type' in self.headers:
            header = self.headers['Content-Type']
        else:
            header = Header('Content-Type', 'text/html')
            self.headers.add_header(header)

        match = self._charset_pattern.match(header.value)
        if match:
            header.value = '%s; charset=%s' % (
                match.group(1), value)
        else:
            header.value = '%s; charset=%s' % (
                header.value, value)

    def links(self, links):
        '''
        A helper for setting the "Link" header. To set links, simply do
        something like::

            response.links = {
                'http://google.com/meta.rdf': 'meta'
            }

        Note that this is a setter-only property. Reading from it is not
        valid.
        '''
        buffer_ = []
        for k, v in links.iteritems():
            buffer_.append('<%s>; rel="%s"' % (v, k))
        self.headers['Link'] = ', '.join(buffer_)

    links = property(None, links, None, links.__doc__)

    def render(self, template, params):
        raise NotImplementedError("Response.render() not implemented!")



import sys
from .connection import Connection
from ...errors import FDError
from . import modulemonitor
from ..logger import StdLogger
import select
import socket
import signal


class HTTPServer(object):
    def __init__(self, app, host='localhost', port=8080, listen=5,
            max_read=8092, chunk_size=1024, auto_reload=True,
            logger=StdLogger()):

        self.app = app
        self.host = host
        self.port = port
        self.listen = listen
        self.chunk_size = chunk_size
        self.max_read = max_read
        self.auto_reload = auto_reload
        self.logger = logger

        self.connections = []

        # Setup socket
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

        if self.auto_reload:
            self.module_monitor = modulemonitor.ModuleMonitor(self)

        '''
        self.environment = Environment(
            loader=ChoiceLoader([PackageLoader('gimme', 'templates'),
                FileSystemLoader('templates')]))
        '''

        self.running = True

        # Basic lists needed for select
        self.r_list = [self.socket]
        self.w_list = []
        self.e_list = []

    def _bind_socket(self):
        self.socket.bind((self.host, self.port))
        self.socket.listen(self.listen)

    def _handle_signal(self, signum, frame):
        if signum in (signal.SIGTERM, signal.SIGINT):
            self.stop(True)

    def _setup_signal_handlers(self):
        signal.signal(signal.SIGTERM, self._handle_signal)
        signal.signal(signal.SIGINT, self._handle_signal)

    def start(self):
        try:
            self._bind_socket()
        except socket.error, e:
            self.logger.log_error("Could not start HTTP Server: %s" %
                e.args[1])
            sys.exit(1)

        self._setup_signal_handlers()

        if self.auto_reload:
            self.module_monitor.start()

        self.logger.log_info("Gimme Asynchronous HTTP Server is now ready")

        while self.running:
            try:
                r_ready, w_ready, e_ready = select.select(self.r_list,
                    self.w_list, self.e_list, 0.01)
            except (select.error, socket.error):
                r_ready, w_ready, e_ready = [], [], []
                self.running = False
            except FDError, e:
                # In this case, a socket has closed prematurely; handle it
                # gracefully and move on
                conn = e.args[0]
                for i in (self.r_list, self.w_list, self.e_list):
                    if conn in i:
                        i.remove(conn)
                r_ready, w_ready, e_ready = [], [], []

            for i in r_ready:
                if i is self.socket:
                    connection = Connection(self, self.socket.accept())
                    self.r_list.append(connection)
                    connection.handle_connect()

                elif isinstance(i, Connection):
                    i.handle_read()

            for i in w_ready:
                i.handle_write()

            for i in e_ready:
                i.handle_error()

    def stop(self, stop_monitor=False):
        self.logger.log_info("Shutting down Gimme HTTP Server...")

        for i in self.connections:
            i.close()
            i.join()

        self.socket.close()

        if stop_monitor and self.auto_reload:
            self.module_monitor.stop()
            self.module_monitor.join()



from .app import App
from .renderers import Json, Template, Compress, Format
from .controller import Controller
from . import middleware
from .errors import HTTPError, HTTPRedirect
from .routes import Route



import abc
from dogpile.cache import make_region


class ChangeTracker(object):
    def __init__(self):
        self._dirty = False

    def is_dirty(self):
        return self._dirty

    def dirty(self):
        self._dirty = True

    def clean(self):
        self._dirty = False


class NotifiedMixin(object):
    def __init__(self):
        if isinstance(self, list):
            i = 0
            for item in self:
                self[i] = self._check(item)
                i += 1
        elif isinstance(self, dict):
            for k, v in self.items():
                self[k] = self._check(v)

    def _check(self, item):
        if isinstance(item, dict) and not isinstance(item, NotifiedDict):
            item = NotifiedDict(self._state, item)
        if isinstance(item, list) and not isinstance(item, NotifiedList):
            item = NotifiedList(self._state, item)
        return item


class NotifiedDict(dict, NotifiedMixin):
    def __init__(self, state, *args, **kwargs):
        self._state = state
        dict.__init__(self, *args, **kwargs)
        NotifiedMixin.__init__(self)

    def __setitem__(self, key, value):
        self._state.dirty()
        value = self._check(value)
        dict.__setitem__(self, key, value)

    def __delitem__(self, key):
        self._state.dirty()
        dict.__delitem__(self, key)


class NotifiedList(list, NotifiedMixin):
    def __init__(self, state, *args, **kwargs):
        self._state = state
        list.__init__(self, *args, **kwargs)
        NotifiedMixin.__init__(self)

    def __delitem__(self, key):
        self._state.dirty()
        list.__delitem__(self, key)

    def append(self, item):
        self._state.dirty()
        item = self._check(item)
        list.append(self, item)

    def remove(self, item):
        self._state.dirty()
        list.remove(self, item)

    def insert(self, index, item):
        self._state.dirty()
        item = self._check(item)
        list.insert(self, index, item)

    def pop(self, *args, **kwargs):
        self._state.dirty()
        return list.pop(self, *args, **kwargs)


class Session(object):
    def __init__(self, cache, key, data={}, new=False):
        self._cache = cache
        self._key = key
        self._state = ChangeTracker()
        self._data = NotifiedDict(self._state, data)

        if new:
            self._state.dirty()

    def __contains__(self, key):
        return key in self._data

    def __getitem__(self, key):
        return self._data[key]

    def __setitem__(self, key, value):
        self._data[key] = value

    def __repr__(self):
        return "Session(%s)" % (self._key)

    def get(self, *args, **kwargs):
        return self._data.get(*args, **kwargs)

    def save(self):
        self._cache.set(self._key, dict(self._data))

    def touch(self):
        self._cache.save()

    def destroy(self):
        self._cache.delete(self._key)



#!/usr/bin/env python

import gimme
import routes


app = gimme.App()

app.use(gimme.middleware.compress())
app.use(gimme.middleware.static('public'))
app.use(gimme.middleware.method_override())
{%- if config.session %}
app.use(gimme.middleware.cookie_parser())
app.use(gimme.middleware.session())
{%- endif %}
app.use(gimme.middleware.body_parser())

routes.setup(app)


if __name__ == '__main__':
    app.listen()



import re


class StatusCode(object):
    _pattern = re.compile('^([0-9]{1,3})\s+(.+?)')

    _status_code_map = {
        100: 'Continue',
        101: 'Switching Protocols',
        102: 'Processing',
        200: 'OK',
        201: 'Created',
        202: 'Accepted',
        203: 'Non-Authoritative Information',
        204: 'No Content',
        205: 'Reset Content',
        206: 'Partial Content',
        207: 'Multi-Status',
        208: 'Already Reported',
        226: 'IM Used',
        300: 'Multiple Choices',
        301: 'Moved Permanently',
        302: 'Found',
        303: 'See Other',
        304: 'Not Modified',
        305: 'Use Proxy',
        306: 'Switch Proxy',
        307: 'Temporary Redirect',
        308: 'Permanent Redirect',
        400: 'Bad Request',
        401: 'Not Authorized',
        402: 'Payment Required',
        403: 'Forbidden',
        404: 'Not Found',
        405: 'Method Not Allowed',
        406: 'Not Acceptable',
        407: 'Proxy Authentication Requred',
        408: 'Request Timeout',
        409: 'Conflict',
        410: 'Gone',
        411: 'Length Required',
        412: 'Precondition Failed',
        413: 'Request Entity Too Large',
        414: 'Request-URI Too Long',
        415: 'Unsupported Media Type',
        416: 'Request Range Not Satisfiable',
        417: 'Expectation Failed',
        418: "I'm a teapot",
        419: 'Authentication Timeout',
        420: 'Enhance Your Calm',
        422: 'Unprocessable Entity',
        423: 'Locked',
        424: 'Failed Dependency',
        425: 'Unordered Collection',
        426: 'Upgrade Required',
        428: 'Precondition Required',
        429: 'Too Many Requests',
        431: 'Request Header Fields Too Large',
        444: 'No Response',
        451: 'Unavailable For Legal Reasons',
        500: 'Internal Server Error',
        501: 'Not Implemented',
        502: 'Bad Gateway',
        503: 'Service Unavailable',
        504: 'Gateway Timeout',
        505: 'HTTP Version Not Supported',
        506: 'Variant Also Negotiates',
        507: 'Insufficient Storage',
        508: 'Loop Detected',
        509: 'Bandwidth Limit Exceeded',
        510: 'Not Extended',
        511: 'Network Authentication Required',
        522: 'Connection Timed Out'
    }

    def __init__(self, status='200 OK'):
        self._status = '200 OK'
        self.set(status)

    def __str__(self):
        return self.get()

    def __repr__(self):
        return "<StatusCode(%s)>" % self.get()

    def __eq__(self, other):
        cls = type(self)
        if not isinstance(other, cls):
            other = cls(other)
        return self.code == other.code

    def __ne__(self, other):
        return not self.__eq__(other)

    def __int__(self):
        return self.code

    def set(self, value):
        if isinstance(value, int):
            self._status = "%s %s" % (value, self._status_code_map[value])
        elif self._pattern.match(value):
            self._status = value
        else:
            raise ValueError("Invalid status code: %s" % value)

    def get(self):
        return self._status

    @property
    def code(self):
        return int(self._status.split(None, 1)[0])

    @property
    def text(self):
        return self._status.split(None, 1)[1]



import os
import sys
import datetime
import socket
import mmap
import re
from collections import deque
from .headers import RequestHeaders, RequestLine
from ...errors import FDError
from ...dotdict import DotDict
from tempfile import NamedTemporaryFile


def parse_body(request):
    try:
        return request.split('\r\n\r\n', 1)[1]
    except IndexError:
        return ''


def populate_format_data(connection, response_length, environ):
    now = datetime.datetime.now()
    timestamp = now.strftime("%d %b %Y %H:%M:%S")

    try:
        status_code, status_message = connection.status.split(None, 1)
    except ValueError:
        status_code = 'INVALID'
        status_message = 'INVALID'

    if 'http_user_agent' in environ:
        user_agent = environ.http_user_agent
    else:
        user_agent = '-'

    if 'referer' in environ:
        referer = environ.referer
    else:
        referer = '-'

    return {
        'remote_host': (environ.remote_addr if
            'remote_addr' in environ else '-'),
        'timestamp': timestamp,
        'request_line': "%s %s %s" % (
            environ.request_method if 'request_method' in environ else '-',
            environ.request_uri if 'request_uri' in environ else '-',
            environ.server_protocol if 'server_protocol' in environ else '-'),
        'status_code': status_code,
        'body_size': response_length,
        'local_address': (environ.server_addr if
            'server_addr' in environ else '-'),
        'environment': environ,
        'request_protocol': (environ.server_protocol if
            'server_protocol' in environ else '-'),
        'request_method': (environ.request_method if
            'request_method' in environ else '-'),
        'server_port': (environ.server_port if
            'server_port' in environ else '-'),
        'query_string': (environ.query_string if
            'query_string' in environ else '-'),
        'request_uri': (environ.request_uri if
            'request_uri' in environ else '-'),
        'server_name': (environ.server_name if
            'server_name' in environ else '-'),
        'user_agent': user_agent,
        'referer': referer
    }


def log_access(connection, response_length, all_headers):
    environ = DotDict({})

    for k, v in all_headers.iteritems():
        environ[k.lower()] = v

    format_data = populate_format_data(connection, response_length, environ)
    print ("%(remote_host)s [%(timestamp)s] \"%(request_line)s\" "
        "%(status_code)s %(body_size)s \"%(referer)s\" \"%(user_agent)s\""
        % format_data)


class Connection(object):
    _headers_pattern = re.compile('^(.{0,8196}\r\n\r\n)', re.S)

    def __init__(self, server, accept):
        self.server = server
        self.socket, self.addr = accept

        self.read_buffer = NamedTemporaryFile()
        self.write_buffer = deque()

        self.total_received = 0
        self.request_body_received = 0
        self.request_body_length = None
        self.request_headers = None
        self.request_body = NamedTemporaryFile()

        # Track number of failures for socket
        self.failures = 0

    def fileno(self):
        if os.name == 'posix':
            try:
                os.fstat(self.socket.fileno())
            except Exception:
                raise FDError(self)
            else:
                return self.socket.fileno()
        else:
            return self.socket.fileno()

    def handle_connect(self):
        pass

    def get_headers(self):
        self.read_buffer.seek(0, 0)
        buf = mmap.mmap(self.read_buffer.fileno(), 0)
        match = self._headers_pattern.search(buf)
        self.read_buffer.seek(0, 2)
        if match:
            return RequestHeaders(match.group(1))

    def handle_read(self):
        try:
            data = self.socket.recv(self.server.chunk_size)
        except socket.error:
            self.fail()
            return

        if not data:
            self.close()
            return

        if not self.request_headers:
            self.read_buffer.write(data)
            request_headers = self.get_headers()
            if request_headers:
                self.read_buffer.seek(len(request_headers._request), 0)
                body_chunk = self.read_buffer.read()
                self.request_body.write(body_chunk)
                self.request_body_received += len(body_chunk)
                self.request_body_length = int(request_headers.get(
                    'HTTP_CONTENT_LENGTH', 0))
                self.request_headers = request_headers
        elif self.request_body_length > self.request_body_received:
            # If headers have already been received, stop appending data to
            # the read_buffer file, but instead add it to the request_body
            # file.
            self.request_body_received += len(data)
            self.request_body.write(data)

        if self.request_body_received >= self.request_body_length:
            self.handle_request()

    def handle_write(self):
        data = self.write_buffer.popleft()

        if data is None:
            self.close()
            return

        try:
            self.socket.send(data)
        except socket.error:
            self.fail()

        #if not self.write_buffer:
        #  self.close()

    def handle_request(self):
        try:
            self.server.r_list.remove(self)
        except ValueError:
            pass

        self.request_body.seek(0)

        wsgi_environ = {
            'wsgi.multiprocess': False,
            'wsgi.url_scheme': 'http',
            'wsgi.input': self.request_body,
            'wsgi.multithread': True,
            'wsgi.version': (1, 0),
            'wsgi.run_once': False,
            'wsgi.errors': ''
        }

        try:
            uri_headers = RequestLine(self.request_headers._request)
        except ValueError:
            self.close()
            return

        other_headers = {
            'SERVER_ADDR': self.server.host,
            'SERVER_PORT': self.server.port,
            'SERVER_NAME': self.server.host,
            'SERVER_PROTOCOL': 'HTTP/1.1',
            'SERVER_SOFTWARE': 'Gimme 0.1.0',
            'REMOTE_ADDR': self.addr[0],
            'REMOTE_PORT': self.addr[1],
            'GATEWAY_INTERFACE': 'CGI/1.1',
            'SCRIPT_FILENAME': os.path.abspath(sys.argv[0]),
            'DOCUMENT_ROOT': os.getcwd() + '/',
            'PATH_TRANSLATED': os.getcwd() + '/' + uri_headers['PATH_INFO'],
            'SCRIPT_NAME': '',
            'REDIRECT_STATUS': 200
        }

        all_headers = dict(self.request_headers.items() + wsgi_environ.items()
            + other_headers.items() + uri_headers.items())

        try:
            response = self.server.app(all_headers, self.start_response)
        except Exception, e:
            response = []
            self.status = '500 Internal Server Error'
            self.headers = [('Content-Type', 'text/html')]

            response.append('<h1>500 Internal Server Error</h1>')

        finally:
            # Send headers
            headers_sent = False
            response_length = 0

            # Send response
            for i in response:
                if not headers_sent:
                    self.send_headers(self.status, self.headers)
                    headers_sent = True
                response_length += len(i)
                log_access(self, response_length, all_headers)
                self.send(i)

            self.write_buffer.append(None)

            if self not in self.server.w_list:
                self.server.w_list.append(self)

    def send_headers(self, status, headers):
        self.send("HTTP/1.1 %s\r\n" % status)
        for k, v in headers:
            self.send("%s: %s\r\n" % (k, v))
        self.send("\r\n")

    def start_response(self, status, headers, other=None):
        self.status = status
        self.headers = headers
        self.other = other

    def send(self, data):
        while data:
            chunk = data[0:self.server.chunk_size]
            self.write_buffer.append(chunk)
            data = data[self.server.chunk_size:]

        # Make sure connection is part of the write list
        #if self not in self.server.w_list:
        #  self.server.w_list.append(self)

    def shutdown(self):
        self.socket.shutdown(socket.SHUT_RDWR)

    def close(self):
        # Deprecated in favor of using e_list
        self.socket.close()

        if self in self.server.r_list:
            self.server.r_list.remove(self)
        if self in self.server.w_list:
            self.server.w_list.remove(self)
        if self in self.server.e_list:
            self.server.e_list.remove(self)

        self.read_buffer.close()
        self.request_body.close()

    def fail(self):
        self.failures += 1
        if self.failures == 5:
            self.close()



import re


class AcceptedParser(object):
    _pattern = re.compile('(?P<value>[^;]+)(?:\s*;\s*q\s*=\s*'
        '(?P<priority>[0-9]*\.?[0-9]*))?')

    def __init__(self, data, value_parser=None):
        self._data = data
        self._value_parser = value_parser
        self._match = self._pattern.match(data)

        if not self._match:
            raise ValueError("Invalid accept type.")

        temp = self._match.groupdict()

        self.value = (
            temp['value'] if not value_parser
            else value_parser(temp['value'])
        )

        self.priority = (
            float(temp['priority'])
            if temp['priority'] is not None else 1
        )

    def __eq__(self, other):
        if isinstance(other, AcceptedParser):
            return self.value == other.value
        else:
            return self.value == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def __gt__(self, other):
        return self.priority > other.priority

    def __lt__(self, other):
        return not self.__gt__(other)

    def __repr__(self):
        return "<AcceptedParser(%s, %s)>" % (self.value, self.priority)

    def __str__(self):
        return str(self.value)


class AcceptedList(list):
    _separator = re.compile('\s*,\s*')

    def __repr__(self):
        return '<AcceptedList(%s)>' % ', '.join(map(str, self))

    def __contains__(self, obj):
        for i in self:
            if i == obj:
                return True
        return False

    def __getslice__(self, start, end):
        return type(self)(list.__getslice__(self, start, end))

    def filter(self, value):
        if not isinstance(value, list):
            value = [value]
        return type(self)([i for i in self if i in value])

    def get_by_priority(self):
        return type(self)(sorted(self, reverse=True))

    def get_highest_priority(self):
        return self.get_by_priority()[0]
            
    @classmethod
    def parse(cls, data, value_parser=None):
        result = []
        split = cls._separator.split(data)
        for i in split:
            try:
                result.append(AcceptedParser(i, value_parser))
            except ValueError:
                continue
        return cls(result)



import re


class ContentType(object):
    _pattern = re.compile('^(?:(?P<category>[a-zA-Z0-9_\-]+|\*)/)?(?P<type>[a-zA-Z0-9_\-]+|\*).*')

    def __init__(self, content_type):
        self._content_type = content_type
        self._match = self._pattern.match(content_type)

        if not self._match:
            raise ValueError("Invalid Content-Type: %s" % content_type)

        self._groups = self._match.groupdict()

        # Some helpers to make accessing data easier
        self._category = self._groups['category']
        self._type = self._groups['type']

    def __eq__(self, other):
        if not isinstance(other, ContentType):
            try:
                other = ContentType(other)
            except ValueError:
                return False

        if other._category is None or self._category is None:
            return (other._type == self._type
                or other._type == '*'
                or self._type == '*')
        elif (other._category == self._category
                or self._category == '*'
                or other._category == '*'):
            if (other._type == '*' or self._type == '*') or (
                    other._type == self._type):
                return True
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        if self._category is None:
            return '<ContentType(%s)>' % self._type
        else:
            return '<ContentType(%s/%s)>' % (self._category, self._type)

    def __str__(self):
        if self._category is None:
            return self._type
        else:
            return '%s/%s' % (self._category, self._type)



import types
from json import dumps as dump_json
from collections import namedtuple
from .parsers.contenttype import ContentType
import errors
import zlib


nt_format = namedtuple('nt_format', 'renderer content_type')


class BaseRenderer(object):
    def __eq__(self, content_type):
        return Format(self, content_type)

    def __repr__(self):
        return "<%s()>" % type(self).__name__


class BulkRenderer(list, BaseRenderer):
    def render(self, controller, data):
        for i in self:
            data = i.render(controller, data)
        return data

    def __repr__(self):
        return "<BulkRenderer([%s])>" % ', '.join(map(repr, self))


class Format(list, BaseRenderer):
    def __init__(self, renderer, content_type='*/*'):
        if not isinstance(content_type, ContentType):
            content_type = ContentType(content_type)
        self.append(nt_format(renderer, content_type))

    def __or__(self, other):
        if isinstance(other, Format):
            for i in other:
                self.append(nt_format(i.renderer, i.content_type))
        else:
            self.append(nt_format(other[0], other[1]))
        return self

    def __repr__(self):
        return "<Format[%s]>" % ', '.join(map(repr, self))

    @property
    def content_types(self):
        result = []
        for i in self:
            result.append(i.content_type)
        return result

    def get_by_content_type(self, content_type):
        for i in self:
            if i.content_type == content_type:
                return i.renderer
        raise IndexError("Could not find content_type: %s" % content_type)

    def render(self, controller, data):
        content_types = self.content_types
        try:
            priority = (controller.request.accepted.filter(content_types)
                .get_highest_priority()).value
        except IndexError:
            priority = self[0].content_type
        renderer = self.get_by_content_type(priority)

        return renderer.render(controller, data)


class Template(BaseRenderer):
    def __init__(self, template_path):
        self.template_path = template_path

    def __repr__(self):
        return "<Template(%s)>" % self.template_path

    def render(self, controller, data):
        app = controller.app
        return app.engine.render(self.template_path, data)


class Json(BaseRenderer):
    def render(self, controller, data):
        controller.response.type = 'application/json'
        return dump_json(data)


class Compress(BaseRenderer):
    def render(self, controller, data):
        if 'accept_encoding' in controller.request.headers:
            if ('deflate' in
                    controller.request.headers.accept_encoding.split(',')):
                try:
                    compressed = zlib.compress(data)
                except TypeError:
                    pass
                else:
                    controller.response.headers['Content-Encoding'] = 'deflate'
                    controller.response.headers['Content-Length'] = str(len(compressed))
                    return compressed
        return data



import contextlib
from .response import Response
from .controller import ErrorController
from .errors import HTTPError


class WSGIAdapter(object):
    def __init__(self, app):
        self.app = app

    def _get_middleware(self, request, response):
        all_middleware = self.app._middleware + response.route.middleware
        return map(lambda x: x(self.app, request, response), all_middleware)

    def _get_app_middleware(self, request, response):
        return map(lambda x: x(self.app, request, response),
            self.app._middleware)

    def process(self, environ, start_response):
        request, response = self.app.routes.match(environ)

        try:
            response._render()
        except Exception, e:
            err = e if isinstance(e, HTTPError) else HTTPError(500)
            err_response = err.make_response(request)
            err_response._render([])

            start_response(str(err_response.status), err_response.headers.items())
            yield str(err_response.body)
        else:
            start_response(str(response.status), response.headers.items())
            yield str(response.body)



import argparse
import os
import stat
from jinja2 import Environment, FileSystemLoader, ChoiceLoader, PackageLoader


def get_args():
    parser = argparse.ArgumentParser(
        description='Code generation tool for Gimme web framework')
    parser.add_argument('-s', '--session', action='store_true', default=False,
        help='Enable session storage.')
    parser.add_argument('appname', help='The name of the application '
        '(also denotes which directory to create the app in).')
    return parser.parse_args()


def make_dir(config, dirname=None):
    path = os.path.join(config.appname, dirname) if dirname else config.appname
    if not os.path.exists(path):
        os.mkdir(path)


def check_dir(config):
    if os.path.exists(config.appname):
        raise StandardError("App directory already exists!")


def render_file(env, config, file_path):
    return env.get_template(file_path).render(config=config)


def save_file(config, file_path, file_contents):
    path = os.path.join(config.appname, file_path)
    if os.path.exists(path):
        raise StandardError("File already exists: %s" % path)
    with open(path, 'w') as f:
        f.write(file_contents)
        trash, ext = os.path.splitext(file_path)
        if ext == '.fcgi':
            st = os.fstat(f.fileno())
            os.fchmod(f.fileno(), st.st_mode | stat.S_IXUSR | stat.S_IXGRP |
                stat.S_IXOTH)


def make_public_dirs(config):
    make_dir(config, 'public')
    make_dir(config, 'public/images')
    make_dir(config, 'public/scripts')
    make_dir(config, 'public/styles')


def main():
    config = get_args()
    check_dir(config)
    gimme_path = os.path.abspath(os.path.dirname(__file__))
    templates_path = os.path.join(gimme_path, 'templates/generator')
    env = Environment(loader=ChoiceLoader([
        FileSystemLoader(templates_path),
        PackageLoader('gimme', 'templates')
    ]))
    files = {}
    make_dir(config)

    for context, dirs, files in os.walk(templates_path):
        for i in dirs:
            full_path = os.path.join(context, i)
            dir_path = full_path[len(templates_path)+1:]
            make_dir(config, dir_path)
        for i in files:
            full_path = os.path.join(context, i)
            file_path = full_path[len(templates_path)+1:]
            trash, ext = os.path.splitext(file_path)
            if ext != '.pyc':
                file_contents = render_file(env, config, file_path)
                save_file(config, file_path, file_contents)

    make_public_dirs(config)


if __name__ == '__main__':
    main()



from urlparse import parse_qs
import re
import urllib


class QueryValue(list):
    def __init__(self, value=[]):
        list.__init__(self, value)

    def __str__(self):
        return self[0]

    def __repr__(self):
        return '<QueryValue(%s)>' % ', '.join(self)

    def __eq__(self, other):
        if isinstance(other, str) and len(self) == 1:
            return self[0] == other
        else:
            return list.__eq__(self, other)

    def __add__(self, other):
        if not isinstance(other, list):
            other = [other]
        copy = list(self)
        return QueryValue(copy + other)

    def __sub__(self, other):
        if not isinstance(other, list):
            other = [other]
        temp = list(self)
        for i in other:
            temp.remove(i)
        return QueryValue(temp)

    def __iadd__(self, other):
        if not isinstance(other, list):
            other = [other]
        return list.__iadd__(self, other)

    def __isub__(self, other):
        if not isinstance(other, list):
            other = [other]
        for i in other:
            self.remove(i)
        return self


class QueryString(object):
    _reserved_attrs = ('_query_string', '_quote_plus', '_parsed')

    def __init__(self, query_string, quote_plus=False):
        self._query_string = query_string
        self._quote_plus = quote_plus
        self._parsed = self._parse_qs(query_string)

    def __repr__(self):
        return 'QueryString(%s)' % self._parsed

    def __bool__(self):
        return bool(self._parsed)

    def __str__(self):
        result = []
        encode = urllib.quote_plus if self._quote_plus else urllib.quote
        for k, v in self._parsed.iteritems():
            for i in v:
                result.append('%s=%s' % (k, encode(i)))
        return '&'.join(result)

    def __getattr__(self, key):
        if key not in QueryString._reserved_attrs:
            try:
                return self._parsed[key]
            except KeyError, e:
                raise AttributeError(key)
        else:
            return object.__getattr__(self, key)

    def __setattr__(self, key, value):
        if key not in QueryString._reserved_attrs:
            if key not in self._parsed:
                self._parsed[key] = QueryValue([value])
        else:
            return object.__setattr__(self, key, value)

    def __getitem__(self, key):
        try:
            return self.__getattr__(key)
        except AttributeError:
            raise KeyError(key)

    def __setitem__(self, key, value):
        self.__setattr__(key, value)

    def __delitem__(self, key):
        del(self._parsed[key])

    def __contains__(self, key):
        return key in self._parsed

    def __iter__(self):
        for i in self._parsed:
            yield i

    def _parse_qs(self, qs):
        temp = parse_qs(qs)
        for k, v in temp.items():
            temp[k] = QueryValue(v)
        return temp
    
    def iteritems(self):
        for i in self:
            yield (i, self[i])

    def items(self):
        return list(self.iteritems())


class URI(object):
    _pattern = re.compile('^([a-zA-Z0-9]+)?:?(\/\/)?([^\/?#]*)([^?]*)\??([^#]+)?#?(.*)$')

    def __init__(self, uri, escape_plus=False):
        self._uri = uri
        self._escape_plus = escape_plus

        match = self._pattern.match(uri)

        if not match:
            raise ValueError("Could not create GetHelper object from this string.")

        self.protocol = match.group(1)
        self.hostname = match.group(3)
        self.path = match.group(4)
        self.query_string = match.group(5) or ''
        self.hash = match.group(6) or ''

        self.query_params = QueryString(self.query_string)
        self.hash_params = QueryString(self.hash)

    def get_query_string(self):
        return str(self.query_params)

    def get_hash_string(self):
        if self.hash_params:
            return str(self.hash_params)
        elif self.hash:
            return self.hash

    def get_uri(self):
        uri = [];
        query_string = self.get_query_string()
        hash_string = self.get_hash_string()

        if self.protocol:
            uri.append(self.protocol + '://')
        elif self.hostname:
            uri.append('//')

        uri.append(self.hostname)
        uri.append(self.path)
        uri.append('?' + query_string if len(query_string) else '')

        if hash_string:
            uri.append('#' + hash_string)

        return ''.join(uri)

    def get_segment(self, i):
        return self.path.split('/')[i]

    def set_segment(self, i, value):
        split = self.path.split('/')
        split[i] = value
        self.path = '/'.join(split)
        return self

    __getitem__ = get_segment
    __setitem__ = set_segment



import os
import sys
from .routes import Routes
from .errors import TemplateError
from .wsgi import WSGIAdapter
from .servers.http import HTTPServer
from .servers.logger import SysLogger
from .middleware import connection_helper
from .engines import Jinja2Engine


class App(object):
    '''
    The central class that ties a Gimme application together.
    
    There are a few configuration parameters that may be passed to App
    objects upon instantiation, although most of the configuration is done
    via middleware.

    Basic usage may look something like the following::

        app = App('some_app')

        # Add middleware.
        app.use(gimme.middleware.session())

        # Add routes to a controller that we have not shown here.
        app.routes.get('/', SomeController.index + 'index.html')
        app.routes.get('/about', SomeController.about + 'about.html')

        # Start the app's development web server.
        app.listen()

    :ivar dirname: Stores the path information from where the app was started
        from.
    :ivar routes: An instance of :class:`gimme.routes.Routes`, which is used
        for mapping routes to controllers, etc.

    :param str name: The name to use for the app in the logger.
    :param engine: The template engine adapter to use.
    :param logger: The log class to use.
    '''

    def __init__(self, name='gimme', engine=Jinja2Engine(), logger=SysLogger):
        self.logger = logger(name)
        self.routes = Routes(self)
        self.engine = engine

        self._middleware = []
        self.__wsgi = WSGIAdapter(self)
        self.__env_config = {}

        self.dirname = os.path.dirname(os.path.abspath(sys.argv[0]))

        # Dictionary to store defined params
        self.__params = {}

        # Dictionary to store app config
        self.__config = {
            'env': 'development',
            'default headers': {
                'Content-Type': 'text/html; charset=UTF-8',
                'X-PoweredBy': 'Blood, sweat, and tears',
                'X-BadIdea': '; DROP TABLE users;'
            }
        }

    def __call__(self, environ, start_response):
        return self.__wsgi.process(environ, start_response)

    def listen(self, port=8080, host='127.0.0.1', http_server=HTTPServer):
        '''
        Starts the built-in development webserver.

        :param int port: The port to listen on.
        :param str host: The hostname/IP address to listen on.
        :param http_server: What class to use for the HTTP server.
        '''
        server = http_server(self, host, port)
        server.start()

    def use(self, middleware):
        '''
        Add middleware to the app::


            app.use(gimme.middleware.body_parser())
            app.use(gimme.middleware.session())

        For more information, see :class:`gimme.middleware.Middleware`.

        :param middleware: The middleware to add to the app.
        '''
        self._middleware.append(middleware)

    def set(self, key, value):
        '''
        Sets a config value. These key/value pairs can be used to store
        arbitrary information. Retrieve via :meth:`get() <gimme.app.App.get>`.

        :param key: The key to store under.
        :param value: The value to store.
        '''
        self.__config[key] = value

    def get(self, key):
        '''
        Gets a key from the app, previously set with
        :meth:`set() <gimme.app.App.set>`.

        :param key: The key to fetch
        '''
        return self.__config[key]

    def param(self, name, callback):
        self.params[name] = callback

    def configure(self, env, callback):
        self.__env_config[env] = callback



import controllers


def setup(app):
    app.routes.get('/', controllers.RootController.index + 'root/index.html')
    app.routes.get('*', controllers.RootController.catch_all +
        'root/catch_all.html')



__all__ = ['SysLogger', 'StdLogger']

import abc
import sys
import syslog
import datetime


class BaseLogger(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def _log_message(self, level, message):
        pass

    def log_info(self, message):
        self._log_message(syslog.LOG_INFO, message)

    def log_warn(self, message):
        self._log_message(syslog.LOG_WARNING, message)

    def log_crit(self, message):
        self._log_message(syslog.LOG_CRIT, message)

    def log_notice(self, message):
        self._log_message(syslog.LOG_NOTICE, message)

    def log_error(self, message):
        self._log_message(syslog.LOG_ERR, message)

    def log_emergency(self, message):
        self._log_message(syslog.LOG_EMERG, message)

    def log_alert(self, message):
        self._log_message(syslog.LOG_ALERT, message)

    # Some aliases
    info = log_info
    warn = log_warn
    log_warning = log_warn
    warning = log_warning
    crit = log_crit
    log_critical = log_crit
    critical = log_critical
    notice = log_notice
    error = log_error
    emergency = log_emergency
    alert = log_alert


class SysLogger(BaseLogger):
    def __init__(self, ident='gimme'):
        syslog.openlog(ident, syslog.LOG_PID | syslog.LOG_PERROR)

    def _log_message(self, level, message):
        syslog.syslog(level, message)


class StdLogger(BaseLogger):
    _level_map = {
        syslog.LOG_INFO: 'info',
        syslog.LOG_WARNING: 'warning',
        syslog.LOG_CRIT: 'critical',
        syslog.LOG_NOTICE: 'notice',
        syslog.LOG_ERR: 'error',
        syslog.LOG_EMERG: 'emergency',
        syslog.LOG_ALERT: 'alert'
    }

    def _log_message(self, level, message):
        timestamp = datetime.datetime.now().strftime('%a %b %d %Y %H:%M:%S.%f')
        level_string = self._level_map[level]
        sys.stderr.write("[%s] [%s] %s\n" % (timestamp, level_string, message))



import sys
import traceback
import types
from jinja2 import Environment, PackageLoader

from .renderers import (
    BaseRenderer,
    Template,
    Json,
    Compress,
    Format,
    BulkRenderer
)


class MethodRenderer(list):
    '''
    The ``MethodRenderer`` provides an interface for multiple renderers to all
    manipulate the controller method's output in an orderly fashion.

    It should be noted that the ``MethodRenderer`` class inherits from
    :func:`list`. The first item in the list should always be an instance
    of :class:`gimme.controller.ControllerMethod`. Every other item should
    inherit from :class:`gimme.renderers.BaseRenderer`.
    '''
    def __init__(self, items=[]):
        if len(items) < 1 or not isinstance(items[0], ControllerMethod):
            raise ValueError("First argument to MethodRenderer must be "
                "an instance of ControllerMethod")

        self.im_class = items[0].im_class
        self.__name__ = items[0].__name__
        list.__init__(self, items)

    def __call__(self):
        '''
        Calls the :class:`ControllerMethod <gimme.controller.ControllerMethod>`
        and then calls :meth:`render` on each of the
        :class:`BaseRenderer <gimme.renderers.BaseRenderer>` objects.

        :return: The final output of the execution chain described above.
        '''
        controller = self[0].controller_instance
        data = self[0]()

        for i in self[1:]:
            data = i.render(controller, data)

        return data

    def __add__(self, other):
        '''
        Adds another :class:`BaseRenderer <gimme.renderers.BaseRenderer>` to
        the list.

        :param BaseRenderer other: The renderer to add.
        :return: self
        '''
        if not isinstance(other, BaseRenderer):
            other = Template(other)
        self.append(other)
        return self

    def __repr__(self):
        return "<MethodRenderer([%s])>" % ', '.join(map(str, self))

    def __eq__(self, content_type):
        '''
        Returns a :class:`Format <gimme.renderers.Format>` object with all of
        the renderers in this object inside it. Sets the ``Format`` content
        type to ``content_type``.

        This is typically used in something like the following::

            app.routes.get('/', SomeController.index + (
                (SomeController.index == 'text/plain') |  # <--
                (SomeController.index.json() == 'application/json') |
                (SomeController.index.template('template.html') == 'text/html')
            ))

        :param str content_type: The content type to set the ``Format`` object
            to.
        :return: A :class:`Format <gimme.renderers.Format>` object.
        '''
        return Format(BulkRenderer(list(self[1:])), content_type)

    def template(self, template_path):
        '''
        Adds a :class:`Template <gimme.renderers.Template>` object with the
        provided path to the list.

        :param str template_path: The path to the template
        :return: self
        '''
        self.append(Template(template_path))
        return self

    def json(self):
        '''
        Adds a :class:`Json <gimme.renderers.Json>` object to the list.

        :return: self
        '''
        self.append(Json())
        return self

    def compress(self):
        '''
        Adds a :class:`Compress <gimme.renderers.Compress>` object to the list.

        :return: self
        '''
        self.append(Compress())
        return self


class ControllerMethod(object):
    '''
    ControllerMethod objects wrap regular controller methods, as defined by
    the application programmer.

    In order to allow greater flexibility and expressiveness, all controller
    methods whose name do not start with ``_`` are replaced with instances of
    ``ControllerMethod``. These objects can be called just as the controller
    methods can, but in addition, other methods and operators are made
    available.

    What all this means, in practice, is that you can do things like this in
    your route definitions::

        app.routes.get('/', RootController.index + 'index.html')

    I hope to elaborate on this and how it works in the future.
    '''

    def __init__(self, cls, fn):
        self.im_class = cls
        self.fn = fn
        self.__name__ = fn.__name__

    def __call__(self):
        return self.fn(self.controller_instance)

    def __add__(self, other):
        '''
        Creates a new :class:`gimme.controller.MethodRenderer` object with
        the ``other`` object as the first renderer. Or, if ``other`` is a
        string, instantiates a :class:`gimme.renderers.Template` object with
        the string and passes that to the
        :class:`gimme.controller.MethodRenderer` instead.

        Example::

            method_renderer = SomeController.some_method + gimme.Template('index.html')
            # or, a shortcut:
            method_renderer = SomeController.some_method + 'index.html'

        :param other: Either a string or an instance of
            :class:`gimme.renderers.BaseRenderer`.
        '''
        if not isinstance(other, BaseRenderer):
            other = Template(other)
        return MethodRenderer([self, other])

    def __eq__(self, content_type):
        '''
        Returns a :class:`gimme.renderers.Format` object that responds to
        HTTP Accept headers as specified by ``content_type``.

        Example::

            format_obj = YourController.your_method == 'text/html'

        :param str content_type: The content type to customize format to.
        '''
        return Format(BulkRenderer([]), content_type)

    def template(self, template_path):
        '''
        Returns a :class:`gimme.controller.MethodRenderer` object with a
        :class:`gimme.renderers.Template` renderer directed at the path
        provided by ``template_path``.

        :param str template_path: Where to point the template to.
        '''
        return MethodRenderer([self, Template(template_path)])

    def json(self):
        '''
        Returns a :class:`gimme.controller.MethodRenderer` object with a
        :class:`gimme.renderers.Json` renderer.
        '''
        return MethodRenderer([self, Json()])

    def compress(self):
        '''
        Returns a :class:`gimme.controller.MethodRenderer` object with a
        :class:`gimme.renderers.Compress` renderer.
        '''
        return MethodRenderer([self, Compress()])


class ControllerMeta(type):
    def __init__(mcs, name, bases, attrs):
        for key, value in [(k, v) for (k, v) in attrs.iteritems()
                if not k.startswith('_')]:
            if hasattr(value, '__call__'):
                setattr(mcs, key, ControllerMethod(mcs, value))

        type.__init__(mcs, name, bases, attrs)


class Controller(object):
    '''
    Like most frameworks, the controller is what bridges the gap between the
    model (business logic) and the view. In Gimme, one thing that is a little
    different from the classic MVC methodology is that controllers are not
    directly tied to views. This is explained further in the
    :class:`gimme.routes.Routes` documentation.

    Controllers are instantiated automatically by Gimme on an as-needed basis.
    If a controller's method is needed to fulfill a request, the controller
    will be instantiated and the method called and its data returned. This
    means that controllers are instantiated per request, as opposed to once,
    as the application is started.

    One thing to note is that, while Gimme instantiates controllers
    automatically, in order to perform unit tests, you must instantiate
    controllers in your tests. This can be done like so::

        controller = YourController(your_app, request_obj, response_obj)

    Of course, to do the above, you must have request and response objects.
    To fetch these objects, the easiest way is generally via the
    :meth:`gimme.routes.Routes.match` method::

        request, response = your_app.routes.match(wsgi_environ)

    (Where the ``wsgi_environ`` variable is a dictionary-like object with
    necessary WSGI environ parameters, such as ``PATH_INFO``,
    ``REQUEST_METHOD``, etc.)

    One other thing that may be worth noting is that this class uses a
    custom metaclass that, upon definition, scans for any method whose name
    does not start with a ``_``, and replaces it with an instance of
    :class:`gimme.controller.ControllerMethod`.

    :ivar app: The gimme app.
    :ivar request: The request object.
    :ivar response: The response object.
    '''
    __metaclass__ = ControllerMeta

    def __init__(self, app, request, response):
        self.app = app
        self.request = request
        self.response = response

    def __new__(cls, *args):
        obj = object.__new__(cls, *args)

        for i in dir(obj):
            attr = getattr(obj, i)
            if isinstance(attr, ControllerMethod):
                attr.controller_instance = obj

        return obj


class ErrorController(Controller):
    def __init__(self, *args, **kwargs):
        Controller.__init__(self, *args, **kwargs)
        self.environment = Environment(
            loader=PackageLoader('gimme', 'templates'))

    def http404(self):
        self.response.status = 404

        return self.environment.get_template('errors/404.html').render({
            'headers': self.request.headers,
        })

    def http500(self):
        self.response.status = 500
        e_type, e_value, e_traceback = sys.exc_info()
        traceback.print_exception(e_type, e_value, e_traceback)

        return self.environment.get_template('errors/500.html').render({
            'message': "Oh snap! Something's borked. :(",
            'headers': self.request.headers,
            'traceback': traceback.format_exception(
                e_type,
                e_value,
                e_traceback)
        })

    def generic(self):
        return self.environment.get_template('errors/generic.html').render({
            'status': self.response._status
        })



import re
import mmap
import mimetools
from tempfile import NamedTemporaryFile
import sys
from ..errors import MultipartError


class MultipartHeaders(dict):
    _pattern = re.compile('^([a-zA-Z0-9_\-]+):\s*(.*?)$', re.M)

    def __init__(self, raw_headers):
        self._raw_headers = raw_headers
        self._parse()

    def _parse(self):
        headers = self._pattern.findall(self._raw_headers)
        for key, value in headers:
            self[key] = value.strip()

    def get(self, key, default=None):
        return dict.get(self, key.lower(), default)

    def __getitem__(self, key):
        return dict.__getitem__(self, key.lower())

    def __setitem__(self, key, value):
        return dict.__setitem__(self, key.lower(), value)

    def __delitem__(self, key):
        return dict.__delitem__(self, key.lower())


class MultipartFile(object):
    _headers_pattern = re.compile('^(.*?)\r\n\r\n', re.S | re.M)
    _name_pattern = re.compile('''name=(?P<quote>["'])(.*?)(?P=quote);?''')
    _filename_pattern = re.compile('''filename=(?P<quote>["'])(.*?)(?P=quote);?''')
    _disposition_pattern = re.compile('^([a-zA-Z0-9_\-]+)')

    def __init__(self, boundary, data, pos, endpos):
        self._boundary = boundary
        self._data = data
        self._pos = pos
        self._endpos = endpos
        self.headers, self._headers_length = self._get_headers()
        self.disposition = self._get_disposition()
        
        # Before continuing, check disposition is sane
        if self.disposition not in ('form-data', 'file'):
            raise MultipartError("Invalid disposition type: %s" %
                self.disposition)
        
        self.name = self._get_field_name()
        self.filename = self._get_field_filename()
        self.content_type = self.headers.get('content-type', None)
        self.file = self._make_tempfile()
        
        if self.disposition == 'form-data' and self.content_type is None:
            self.value = self.file.read()
            self.file.seek(0)
        else:
            self.value = None

    def __repr__(self):
        return '<MultipartFile(%s)>' % self.name
        
    def _get_disposition(self):
        disposition = self.headers.get('content-disposition', '')
        match = self._disposition_pattern.search(disposition)
        if match:
            return match.group(1)

    def _get_field_name(self):
        disposition = self.headers.get('content-disposition', '')
        match = self._name_pattern.search(disposition)
        if not match:
            raise MultipartError("Multipart file missing (or invalid) name in "
                "content-disposition")
        return match.group(2)

    def _get_field_filename(self):
        disposition = self.headers.get('content-disposition', '')
        match = self._filename_pattern.search(disposition)
        if match:
            return match.group(2)

    def _make_tempfile(self):
        transfer_encoding = self.headers.get('content-transfer-encoding',
            '').lower()
        tf = NamedTemporaryFile()
        start_pos = self._pos + self._headers_length + 2
        file_length = (self._endpos - 2) - start_pos
        bytes_read = 0

        self._data.seek(start_pos)

        while bytes_read < file_length:
            remaining_bytes = (self._endpos - 2) - self._data.tell()
            chunk_size = min(8196, remaining_bytes)
            tf.write(self._data.read(chunk_size))
            bytes_read += chunk_size

        tf.seek(0)

        if transfer_encoding not in ('', '7bit', '8bit', 'binary'):
            decoded_tf = NamedTemporaryFile()
            mimetools.decode(tf, decoded_tf, transfer_encoding)
            decoded_tf.seek(0)
            return decoded_tf
        else:
            return tf

    def _get_headers(self):
        match = self._headers_pattern.search(self._data, self._pos,
            self._endpos)
        if not match:
            raise MultipartError("Invalid multipart headers.")
        raw_headers = self._data[match.start():min(match.end(), 8196)]
        return MultipartHeaders(raw_headers), len(raw_headers)


class MultipartParser(dict):
    def __init__(self, boundary, fileobj, chunk_size=8096):
        self.boundary = boundary
        self.fileobj = fileobj
        self.chunk_size = chunk_size
        dict.__init__(self, self._parse())

    def _split_files(self):
        files = []
        self.fileobj.seek(0)

        pattern = re.compile('(?<=--{0})(.*?)(?=--{0})'.format(
            re.escape(self.boundary)), re.S)
        filedata = mmap.mmap(self.fileobj.fileno(), 0)

        for i in pattern.finditer(filedata):
            try:
                files.append(MultipartFile(self.boundary, filedata, i.start(),
                    i.end()))
            except MultipartError:
                continue

        filedata.close()

        return files

    def _parse(self):
        for i in self._split_files():
            yield i.name, i



import sys
import os
import abc
from jinja2 import Environment, PackageLoader, FileSystemLoader, ChoiceLoader


class BaseEngine(object):
    '''
    The base class that all engines should derive from.
    '''

    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def render(self, template, data):
        '''
        Returns a rendered template.

        :param template: The template to render (usually a filename/path).
        :param data: The data to send to the template engine. Usually a dict.
        '''
        pass


class Jinja2Engine(BaseEngine):
    '''
    Provides Jinja2 integration.

    :param template_path: The path, relative the current working directory,
        that Jinja2 should look for templates.
    :param environment: A Jinja2 :class:`Environment <jinja2.Environment>`
        object. Defaults to ``None``. If provided, ``template_path`` is
        ignored.
    '''

    def __init__(self, template_path='views', environment=None):
        app_path = os.path.dirname(os.path.abspath(sys.argv[0]))
        if not environment:
            environment = Environment(loader=ChoiceLoader([
                FileSystemLoader(os.path.join(app_path, template_path)),
                PackageLoader('gimme', 'templates')
            ]))
        self.environment = environment

    def render(self, template, data):
        return self.environment.get_template(template).render(data)



from setuptools import setup, find_packages


setup(
    name='gimme',
    version='0.1.3',
    packages=[
        'gimme',
        'gimme.ext',
        'gimme.servers',
        'gimme.servers.http',
        'gimme.servers.logger',
        'gimme.parsers'
    ],
    package_data={
        'gimme': [
            'templates/errors/*.html',
            'templates/generator/*.py',
            'templates/generator/*.fcgi',
            'templates/generator/controllers/*.py',
            'templates/generator/public/*.*',
            'templates/generator/public/images/*.*',
            'templates/generator/public/scripts/*.*',
            'templates/generator/public/styles/*.*',
            'templates/generator/views/*.*',
            'templates/generator/views/root/*.*'
        ]
    },
    entry_points={
      'console_scripts': [
        'gimme = gimme.generator:main'
      ],
      'dogpile.cache': [
        'gimme.cache.memory = gimme.cache:Memory',
        'gimme.cache.file = gimme.cache:File',
        'gimme.cache.redis = gimme.cache:Redis'
      ]
    },
    author='Tim Radke',
    author_email='countach74@gmail.com',
    download_url='https://github.com/countach74/gimme/tarball/master',
    description='An ExpressJS-inspired web framework... in Python',
    license='MIT',
    test_suite='tests',
    install_requires=['jinja2', 'dogpile.cache'],
)



from .dotdict import DotDict


class Header(object):
    def __init__(self, key, value):
        self.key = key
        self.value = value

    def __str__(self):
        return "%s: %s" % (self.key, self.value)

    def __repr__(self):
        return "<Header(%s)>" % str(self)

    def __eq__(self, other):
        if isinstance(other, str):
            return self.value == other
        else:
            return self is other


class HeadersDict(object):
    def __init__(self, initial_data=None):
        self._headers = []

        if initial_data:
            self.update(initial_data)

    def __setitem__(self, key, value):
        self._set_header(key, value)

    def __getitem__(self, key):
        header = self._get_header(key)
        if not header:
            raise KeyError(key)
        return header

    def __delitem__(self, key):
        self._headers.remove(self[key])

    def __contains__(self, key):
        return bool(self._get_header(key))

    def __len__(self):
        return len(self._headers)

    def __repr__(self):
        temp = {}
        for i in self._headers:
            temp[i.key] = i.value
        return repr(temp)

    def __str__(self):
        return self.render()

    def __iter__(self):
        for i in self._headers:
            yield i

    def __eq__(self, other):
        for i in xrange(len(self)):
            try:
                if self._headers[i] != other._headers[i]:
                    return False
            except IndexError:
                return False
        return True

    def _set_header(self, key, value):
        header = self._get_header(key)
        if not header:
            header = Header(key, value)
            self._headers.append(header)
        else:
            header.value = value
        return self

    def _get_header(self, key):
        for header in self._headers:
            if header.key == key:
                return header
        return None

    def keys(self):
        keys = [i.key for i in self._headers]
        return list(set(keys))

    def values(self):
        return [i.value for i in self._headers]

    def items(self):
        return [(i.key, i.value) for i in self._headers]

    def iteritems(self):
        for i in self._headers:
            yield (i.key, i.value)

    def update(self, data):
        for key, value in data.iteritems():
            self[key] = value

    def add_header(self, header):
        self._headers.append(header)

    def del_header(self, header):
        self._headers.remove(header)

    def render(self):
        return '\r\n'.join(map(str, self._headers)) + '\r\n\r\n'

    def copy(self):
        copy = HeadersDict()
        copy._headers = list(self._headers)
        return copy

    def clear(self):
        del(self._headers[:])

    def get(self, key, default=None):
        return self[key] if key in self else default

    def get_all(self, key):
        headers = []
        for i in self:
            if i.key == key:
                headers.append(i)
        return headers


class RequestHeaders(DotDict):
    pass


class ResponseHeaders(HeadersDict):
    pass



import re
from .headers import RequestHeaders
from .dotdict import DotDict
from .uri import QueryString
from .errors import AcceptFormatError
from .parsers.contenttype import ContentType
from .parsers.accepted import AcceptedList


class Request(object):
    '''
    The Request class is responsible for parsing the request headers and body
    into Pythonic shindigs that are (supposed to be) very easy to work with.

    :ivar app: The Gimme application.
    :ivar environ: The WSGI environ dict.
    :ivar headers: The request headers (an instance of :class:`gimme.headers.RequestHeaders`).
    :ivar wsgi: The WSGI headers. Also an instance of :class:`gimme.headers.RequestHeaders`.
    :ivar params: A :class:`gimme.dotdict.DotDict` of the parameters from the
        route URI.
    :ivar query: The query string in a dictionary-like object.
    :ivar accepted: A handy parsing of the HTTP "Accept" header. An instance
        of :class:`gimme.parsers.accepted.AcceptedList`.
    :ivar accepted_languages: A handy parsing of the HTTP "Accept-Language"
        header. An instance of :class:`gimme.parsers.accepted.AcceptedList`.
    :ivar accepted_charsets: A handy parsing of the HTTP "Accept-Charset"
        header. An instance of :class:`gimme.parsers.accepted.AcceptedList`.
    :ivar cookies: The raw HTTP "Cookie" string.
    :ivar type: Either an instance of
        :class:`gimme.parsers.contenttype.ContentType` if the HTTP header
        "Content-Type" is present or ``None``.

    :param app: The Gimme application.
    :param environ: The WSGI environ dict.
    :param match: The matching route, if any.
    '''
    _host_pattern = re.compile('^([^:]*)(:[0-9]+)?')

    def __init__(self, app, environ, match=None):
        self.app = app
        self.environ = environ
        self.headers = RequestHeaders()
        self.wsgi = RequestHeaders()
        self.params = DotDict(match.match.groupdict() if match else {})
        self.__raw_body = None

        self._populate_headers(environ)

        self.query = QueryString(self.headers.get('query_string', ''))

        self.accepted = AcceptedList.parse(
            self.headers.get('accept', ''), ContentType)

        self.accepted_languages = AcceptedList.parse(
            self.headers.get('accept_language', ''))

        self.accepted_charsets = AcceptedList.parse(
            self.headers.get('accept_charset', ''))

        self.cookies = self.headers.get('cookie', '')
        self.type = (ContentType(self.headers.content_type) if 'content_type'
            in self.headers else None)

    def _populate_headers(self, environ):
        for k, v in environ.iteritems():
            key = k.lower()
            if key.startswith('http_'):
                self.headers[key[5:]] = v
            elif key.startswith('wsgi.'):
                self.wsgi[key[5:]] = v
            else:
                self.headers[key] = v

    def get(self, key):
        '''
        Return the specified request header.

        :param key: The header to return.
        '''
        return self.headers[key.replace('-', '_')]

    def accepts(self, content_type):
        '''
        Tests if a given content type is in the HTTP "Accept" header.

        :param content_type: The content type to check for.
        '''
        return content_type in self.accepted

    def accepts_language(self, language):
        '''
        Tests if a given language is in the HTTP "Accept-Language" header.

        :param language: The language to check for.
        '''
        return language in self.accepted_languages

    def accepts_charset(self, charset):
        '''
        Tests if a given charset is in the HTTP "Accept-Charset" header.

        :param charset: The charset to check for.
        '''
        return charset in self.accepted_charsets

    @property
    def raw_body(self):
        '''
        The raw request body.
        '''
        if self.__raw_body is None:
            if (self.headers.get('request_method', None) in ('PUT', 'POST')
                    and 'content_length' in self.headers):
                content_length = int(self.headers.content_length)
                self.__raw_body = self.wsgi.input.read(content_length)
            else:
                self.__raw_body = ''
        return self.__raw_body

    def param(self, key):
        '''
        A simple helper method that looks for a given key in three places:
        self.params, self.body, and self.query.

        :param key: The parameter to look for.
        '''
        if key in self.params:
            return self.params[key]
        elif hasattr(self, 'body') and key in self.body:
            return self.body[key]
        elif key in self.query:
            return self.query[key]
        raise KeyError(key)

    @property
    def xhr(self):
        '''
        Whether or not the HTTP header "X-Requested-With" is present and
        set to "XMLHttpRequest".
        '''
        return self.headers.get('x_requested_with', None) == 'XMLHttpRequest'

    @property
    def path(self):
        '''
        A shortcut for the "PATH_INFO" environ parameter.
        '''
        return self.headers.get('path_info', None)

    @property
    def host(self):
        '''
        A shortcut for the HTTP "Host" header.

        *Note: Strips any port off of the header.*
        '''
        raw_host = self.headers.get('host', '')
        return self._host_pattern.match(raw_host).group(1)

    @property
    def subdomains(self):
        '''
        A list of subdomains per the ``host`` attribute.
        '''
        split = self.headers.get('host', '').split('.')
        return split[:-2] if len(split) > 2 else []

    @property
    def ip(self):
        '''
        A shortcut for the "REMOTE_ADDR" environ parameter.
        '''
        return self.headers.get('remote_addr', None)

    @property
    def secure(self):
        '''
        Right now, Gimme only runs in standard HTTP mode. SSL should be
        implemented at the HTTP server end and piped to Gimme via FastCGI.
        '''
        return False

    @property
    def original_url(self):
        '''
        A shortcut for the "REQUEST_URI" environ parameter.
        '''
        return self.headers.get('request_uri', None)

    @property
    def protocol(self):
        '''
        Gimme only supports HTTP as of the time of this writing.
        '''
        return 'http'



from dogpile.cache.api import CacheBackend, NO_VALUE
import pickle
import os


class Memory(CacheBackend):
    def __init__(self, arguments):
        self.cache = {}

    def get(self, key):
        return self.cache.get(key, NO_VALUE)

    def set(self, key, value):
        self.cache[key] = value

    def delete(self, key):
        del(self.cache[key])


class Redis(CacheBackend):
    def __init__(self, arguments):
        if 'redis' not in arguments:
            raise KeyError("Redis cache backend expects 'redis' argument.")
        self.redis = arguments['redis']
        self.prefix = arguments.get('prefix', 'gimme_session::')
        self.expiration_time = arguments.get('redis_expiration_time', 60*60*24*7)

    def get(self, key):
        value = self.redis.get(self.prefix + key)
        return NO_VALUE if value is None else pickle.loads(value)

    def set(self, key, value):
        self.redis.set(self.prefix + key, pickle.dumps(value),
            ex=self.expiration_time)

    def delete(self, key):
        self.redis.delete(key)


class File(CacheBackend):
    def __init__(self, arguments):
        self.directory = arguments.get('directory', '/tmp/gimme')

    def get(self, key):
        try:
            with open(os.path.join(self.directory, key), 'r') as f:
                return pickle.load(f)
        except:
            return NO_VALUE

    def set(self, key, value):
        path = os.path.join(os.path.abspath(self.directory), key)

        if not os.path.exists(os.path.abspath(self.directory)):
            os.makedirs(os.path.abspath(self.directory))
        try:
            with open(path, 'w') as f:
                pickle.dump(value, f)
        except:
            pass

    def delete(self, key):
        try:
            os.unlink(os.path.join(self.directory, key))
        except OSError:
            pass



import re
import sys
import os


class RequestMixin(object):
    def __init__(self, request):
        self._request = request
        self._data = self._parse(request)


class Headers(object):
    def __getitem__(self, key):
        return self._data[key]

    def __contains__(self, key):
        return key in self._data

    def __iter__(self):
        for i in self._data:
            yield i

    def _format_key(self, key):
        return key.replace("-", "_").upper()

    def items(self):
        return self._data.items()

    def get(self, *args, **kwargs):
        return self._data.get(*args, **kwargs)


class RequestHeaders(Headers, RequestMixin):
    __request_pattern = re.compile("([a-zA-Z0-9\-]+):\s+(.*)")

    def _parse(self, request):
        result = {}
        data = re.findall(self.__request_pattern, request.strip())

        for k, v in data:
            result["HTTP_%s" % self._format_key(k)] = v.strip()
        return result

    def __repr__(self):
        return "<RequestHeaders(%s)>" % self._data


class RequestLine(Headers, RequestMixin):
    __request_line_pattern = re.compile(
        "^([A-Z]+) ([\S]+) HTTP/([0-1\.]+)\r\n")

    def __repr__(self):
        return "<RequestLine('%s')>" % self._request

    def _parse(self, request):
        match = re.match(self.__request_line_pattern, request)

        if not match:
            raise ValueError("Invalid request line.")

        uri_split = match.group(2).split('?', 1)
        path_info = uri_split[0]
        query_string = uri_split[1] if len(uri_split) > 1 else ''

        return {
            'REQUEST_METHOD': match.group(1),
            'REQUEST_URI': match.group(2),
            'PATH_INFO': path_info,
            'QUERY_STRING': query_string,
        }
